'''
###   comgadfly.py
###   An COM object for the Gadfly RDBMS (http://www.chordate.com)
###
###   Marc Risney (msrisney@hotmail.com) -- 1/29/20000
###     http://wwww.blazesoft.com/~mrisney/comgadfly.py

###   comgadfly is a simple COM object  for Gadfly, allowing access to 
###   the Gadfly RDBMS, much like the ADO object, only the real beauty
###   of a Python Gadfly COM object is that you can access you data across the network
###   on a platform that Python and Gadfly is running and being served. 

###   This is still very new code, with only very basic functionality
###   look for a more complete version of this sometime this spring 2000. 

###   Make sure you have Mark Hammonds win32 extension installed

###  Thanks to:
###      Aaron Watters!
###      Jeff Berliner
###      Mark Hammond
###
###   Usage:
###   >>>python comgadfly.py
###      this will register  comgadfly.Connect COM object
###   >>>python comgadfly.py /unreg
###      this will unregister the COM object
###   >>>python comgadfly.py /test
###      basic test against Aaron's supplied test data
###
###   finally here is some sample VBScript to test the COM gadfly object.
###   
###   Dim DB 
###   Dim sql
###   
###   Set DB = CreateObject("Gadfly.Connection")
###       DB.Open "admin",2222,"admin","localhost" 
###	   
###       sql = "select * from __table_names__"
###       DB.Execute(sql)      
###		  Do While Not DB.EOF
###    			msgbox(DB.Field())
###		  DB.MoveNext
###		  Loop
'''


from win32com.server.exception import COMException
import win32com.server.util
import win32com.client.dynamic
import gadfly
import sys
import string

debugging = 0
if debugging:
    from win32com.server.dispatcher import DefaultDebugDispatcher
    useDispatcher = DefaultDebugDispatcher
else:
    useDispatcher = None
 

class Connection:
	_reg_clsid_ = "{566968A0-D543-11D3-9455-0060088978BB}"
	_reg_desc_ = "Gadfly COM Connection"
	_reg_progid_ = "Gadfly.Connection"
	_reg_class_spec_ = "comgadfly.Connection"
	_public_methods_ = ['Open','Close','Execute','Field','MoveNext','Create']
	_public_attrs_ = ['EOF','BOF']
	_readonly_attrs_ = ['author','version']
	
	def __init__(self):
		self.author = 'Marc Risney'
		self.version = '0.51 beta'
		self.db = None
		self.cursor = None
		self.recordset = None
		
		#--- this is the ugly stuff, I gladly welcome someone to do this right
		self.BOF = None		# not yet implemeted 
		self.EOF = None		# a hack, don't laugh it works...
		self.fp = 0		# my weak ass attempt at a file pointer			 
	
	
	def Open(self,*args):
		if len(args) < 2:
			print "Enter database name (*.gfd), followed by the directory path."
		elif len(args) == 2:
			db = str(args[0])
			dir = str(args[1])
			try:
				self.db = gadfly.gadfly(db,dir)
			except IOError, msg: 
				print 'Unable to locate Gadfly database "%s" at location "%s"' %(db,dir)
			else:
				print 'Connected to "%s" at location "%s"'%(db,dir)
				self.cursor = self.db.cursor()
			
				
		elif len(args) > 2: 
			policy = str(args[0]) # the name of the connection policy ["admin" for admin]
			port = int(args[1])
			passwd = str(args[2])
			machine = str(args[3])
			try:
				from gfclient import gfclient
				self.db = gfclient(policy, int(port), passwd, machine)
			except IOError, msg: 
				print 'Unable to connect to database "%s" on port "%s" using "%s" on #%s"' %(db,str(port),passwd,machine)	
			else:
				print 'Connected as "%s" policy on location "%s"'%(policy,machine)
				self.cursor = self.db.cursor()
				

	def Close(self):
		self.db.close()
		
	def Create(self,dbase,loc):
		self.db = gadfly.gadfly()
		try:
			dbase = str(dbase)
			loc = str(loc)
			self.db.startup(dbase,loc)
			self.db.commit()
		except:
			print 'unable to create "%s" gadfly database in location "%s"'%(db,loc)
	
	def Execute(self,sql):
		sqlstmnt = str(sql)
		if string.upper(sqlstmnt[0:6]) == 'SELECT':
			self.cursor.execute(sqlstmnt)
			self.recordset = self.cursor.fetchall()
			self.EOF = max(range(len(self.recordset))) + 1
			return str(self.recordset[self.fp])
		elif string.upper(sqlstmnt[0:6]) == 'COMMIT':	
			self.db.commit()
			return "Commit complete"
		elif string.upper(sqlstmnt[0:6]) == 'INSERT': 		
			self.cursor.execute(sqlstmnt)
			return "1 row created"
		elif string.upper(sqlstmnt[0:6]) == 'CREATE':
			self.cursor.execute(sqlstmnt)
		elif string.upper(sqlstmnt[0:8]) == 'ROLLBACK':
			self.db.rollback()
			return "Rollback complete"
		elif string.upper(sqlstmnt[0:6]) == 'DELETE':
			self.cursor.execute(sqlstmnt)
		elif string.upper(sqlstmnt[0:6]) == 'UPDATE':
			self.cursor.execute(sqlstmnt)
		elif string.upper(sqlstmnt[0:4]) == 'DROP':
			self.cursor.execute(sqlstmnt)	
			
		
			
		
 	
		
	def Field(self,arg=None):
		try: # tests to see if the user has chosen a numerical index
			index = arg
			if index >= 0:
				data = self.recordset[self.fp]
				return str(data[index])
		except:
			try:# assuming the user has instead specified the column by name.
				# I have a feeling performance is going down the toilet here...
				field = str(arg)
				desc = self.cursor.description
				temp = {}
				x = 0
				names = []
				for col in desc:
					n = col[0]
					names.append(n)
					temp[n] = x
					x = x+1
				fieldInt = int(temp[field])
				data = self.recordset[self.fp]
				return str(data[fieldInt])
				
			except:
				return str(arg) + " column not found in current recordset"
	
	def MoveNext(self):
			# this is not pretty,somebody tell me that they are my friggin' daddy here...
			# take over and do it right ....pullleeeazze!
			self.fp = self.fp + 1
			self.EOF = self.EOF - 1
			if self.EOF == 0:
				self.EOF = None
	
	
	
		
def Register():
	import win32com.server.register
	win32com.server.register.UseCommandLine(Connection,debug=debugging)
	print "Gadfly COM server connection successfully registered"
	
def UnRegister():
	from win32com.server.register import UnregisterServer
	UnregisterServer(Connection()._reg_clsid_)
	print "Gadfly COM server connection successfully unregistered"
	

	
def TestCOM():
	try:
		ds = win32com.client.dynamic.Dispatch("Gadfly.Connection")
		# Gadfly.Connect supports both connection to a server and local *.gfd datafiles
		# Depending upon your Gadfly setup or if you are serving,
		# change test parameters here
		#ds.Open("admin",222,"admin","localhost")
		ds.Open("test","D:\Python\Gadfly\dbtest")
		sql = "create table test3(col1 varchar, col2 varchar)"
		ds.Execute(sql)
		ds.Execute("insert into test3 values('test','test2')")
		ds.Execute("commit")
		#field = ds.Field("BEER")
		#print field

		
	except IOError, msg:
		print msg

	
	
if __name__=='__main__':
	if "/unreg" in sys.argv:
		UnRegister()
	elif "/test" in sys.argv:
		print "Testing COM"
		TestCOM()
	else:
		Register()
	
